package au.csiro.cmis.sact.xmlblaster;

import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xmlBlaster.client.I_Callback;
import org.xmlBlaster.client.SubscribeKeyWrapper;
import org.xmlBlaster.client.SubscribeQosWrapper;
import org.xmlBlaster.client.UpdateKey;
import org.xmlBlaster.client.UpdateQos;
import org.xmlBlaster.client.protocol.XmlBlasterConnection;
import org.xmlBlaster.util.XmlBlasterException;

/**
 * <p>Title: Spread Benchmarks</p>
 * <p>Description: Test spread (http://www.spread.org) performance</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CSIRO</p>
 * @author Doug Palmer &lt;Doug.Palmer@csiro.au&gt;
 * @version 1.0
 */

public class Receiver implements I_Callback {
  // The listener for this
  private class ReceiverListener implements I_Callback {
    public String update(String cbSessionId, UpdateKey key, byte[] content, UpdateQos qos) {
      Receiver.this.count++;
      return "";
    }
  }

  // The name of the sender
  private String name;
  // The group to join
  private String groupName;
  // The quality of service
  private String qos;
  // The spread connection
  private XmlBlasterConnection connection;
 // The xmlBlaster subscribe key
  private SubscribeKeyWrapper subscribeKey;
  // The xmlBlaster publish qos
  private SubscribeQosWrapper subscribeQos;
  // The listener
  private ReceiverListener listener;
  // The message count
  private int count;

  /**
   * Constructor
   *
   * @param name The name of the sender
   * @param groupName The group name to send to
   * @param server The hostname of the daemon to connect to
   * @param port The port number to connect to
   */
  public Receiver(String name, String groupName, String qos) {
    this.name = name;
    this.groupName = groupName;
    this.qos = qos;
  }

  /**
   * Connect this sender to the spread daemon
   */
  public void connect() throws XmlBlasterException {
    if (this.connection == null) {
      this.listener = new ReceiverListener();
      this.connection = new XmlBlasterConnection();
      this.connection.connect(null, this);
      this.subscribeKey = new SubscribeKeyWrapper(this.groupName);
      this.subscribeQos = new SubscribeQosWrapper();
    }
  }

  /**
   * Disconnect from the spread daemon
   */
  public void disconnect() throws XmlBlasterException {
    if (this.connection != null) {
      this.connection.disconnect(null);
      this.connection = null;
    }
  }

  /**
   * Start receiving messages
   */
  public void start() throws XmlBlasterException {
      this.connection.subscribe(this.subscribeKey.toXml(), this.subscribeQos.toXml(), this.listener);
  }

  /**
   * Stop receiving messages
   */
  public void stop() throws XmlBlasterException {
    this.connection.unSubscribe(this.subscribeKey.toXml(), this.subscribeQos.toXml());
  }

  /**
   * Get the count of messages
   *
   * @return The number of messages received
   */
  public int getCount() {
    return this.count;
  }

  /**
   * The default callback for this subscription.
   * <P>
   * If this is called, then there is an error afoot.
   * However, xmlBlaster seems to send all messages to everyone.
   * No matter what's subscribed.
   */
  public String update(String cbSessionId, UpdateKey key, byte[] content, UpdateQos qos) {
      // System.err.println("Receieved odd message in session " + cbSessionId + " key=" + key.toString());
      return "";
  }
}